"""Implementation of SystemLink Feeds Client."""

from typing import BinaryIO, List, Optional

from nisystemlink.clients import core
from nisystemlink.clients.core._uplink._base_client import BaseClient
from nisystemlink.clients.core._uplink._methods import delete, get, post
from uplink import Part, Path, Query, retry

from . import models


@retry(when=retry.when.status(429), stop=retry.stop.after_attempt(5))
class FeedsClient(BaseClient):
    def __init__(self, configuration: Optional[core.HttpConfiguration] = None):
        """Initialize an instance.

        Args:
            configuration: Defines the web server to connect to and information about
                how to connect. If not provided, the
                :class:`HttpConfigurationManager <nisystemlink.clients.core.HttpConfigurationManager>`
                is used to obtain the configuration.

        Raises:
            ApiException: if unable to communicate with the Feeds Service.
        """
        if configuration is None:
            configuration = core.HttpConfigurationManager.get_configuration()

        super().__init__(configuration, base_path="/nifeed/v1/")

    @post("feeds")
    def create_feed(self, feed: models.CreateFeedRequest) -> models.Feed:
        """Create a new feed with the provided feed details.

        Args:
            feeds (models.CreateFeedsRequest): Request to create the feed.

        Returns:
            models.Feed: Details of the created feed.

        Raises:
            ApiException: if unable to communicate with the Feeds Service.
        """
        ...

    @get("feeds", args=[Query, Query])
    def __query_feeds(
        self,
        platform: Optional[str] = None,
        workspace: Optional[str] = None,
    ) -> models.QueryFeedsResponse:
        """Lists available feeds for the Platform `platform` under the Workspace `workspace`.

        Args:
            platform (Optional[str]): Information about system platform. Defaults to None.
            workspace (Optional[str]): Workspace id. Defaults to None.

        Returns:
            models.QueryFeedsResponse: List of feeds.

        Raises:
            ApiException: if unable to communicate with the Feeds Service.
        """
        ...

    def query_feeds(
        self,
        platform: Optional[models.Platform] = None,
        workspace: Optional[str] = None,
    ) -> List[models.Feed]:
        """Lists available feeds for the Platform `platform` under the Workspace `workspace`.

        Args:
            platform (Optional[models.Platform]): Information about system platform.
                Defaults to None.
            workspace (Optional[str]): Workspace id. Defaults to None.

        Returns:
            List[models.Feed]: List of feeds.

        Raises:
            ApiException: if unable to communicate with the Feeds Service.
        """
        platform_by_str = platform.value if platform is not None else None
        response = self.__query_feeds(
            platform=platform_by_str,
            workspace=workspace,
        ).feeds

        return response

    @post(
        "feeds/{feedId}/packages",
        args=[Path(name="feedId"), Part(), Query(name="shouldOverwrite")],
    )
    def __upload_package(
        self,
        feed_id: str,
        package: Part,
        overwrite: Query = False,
    ) -> models.Package:
        """Upload package to SystemLink feed.

        Args:
            feed_id (str): ID of the feed.
            package (Part): Package file to be uploaded.
            overwrite (Query): Set to True, to overwrite the package if it already exists.
                Defaults to False.

        Returns:
            models.Package: Uploaded package information.

        Raises:
            ApiException: if unable to communicate with the Feeds Service.
        """
        ...

    def upload_package(
        self,
        feed_id: str,
        package_file_path: str,
        overwrite: bool = False,
    ) -> models.Package:
        """Upload package to SystemLink feed.

        Args:
            feed_id (str): ID of the feed.
            package_file_path (str): File path of the package to be uploaded.
            overwrite (bool): Set to True, to overwrite the package if it already exists.
                Defaults to False.

        Returns:
            models.Package: Uploaded package information.

        Raises:
            ApiException: if unable to communicate with the Feeds Service.
            OSError: if the file does not exist or cannot be opened.
        """
        response = self.__upload_package(
            feed_id=feed_id,
            overwrite=overwrite,
            package=open(package_file_path, "rb"),
        )

        return response

    def upload_package_content(
        self,
        feed_id: str,
        package: BinaryIO,
        overwrite: bool = False,
    ) -> models.Package:
        """Upload package to SystemLink feed.

        Args:
            feed_id (str): ID of the feed.
            package (BinaryIO): Package file to be uploaded.
            overwrite (bool): Set to True, to overwrite the package if it already exists.
                Defaults to False.

        Returns:
            models.Package: Uploaded package information.

        Raises:
            ApiException: if unable to communicate with the Feeds Service.
        """
        response = self.__upload_package(
            feed_id=feed_id,
            overwrite=overwrite,
            package=package,
        )

        return response

    @delete(
        "feeds/{feedId}",
        args=[Path(name="feedId")],
    )
    def delete_feed(self, id: str) -> None:
        """Delete feed and its packages.

        Args:
            id (str): ID of the feed to be deleted.

        Returns:
            None.

        Raises:
            ApiException: if unable to communicate with the Feeds Service.
        """
        ...



from typing import List, Optional

from nisystemlink.clients.core import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel


class DeleteProductsPartialSuccess(JsonModel):
    """The result of deleting multiple products when one or more products could not be deleted."""

    ids: List[str]
    """The IDs of the products that were successfully deleted."""

    failed: Optional[List[str]] = None
    """The IDs of the products that could not be deleted."""

    error: Optional[ApiError] = None
    """The error that occurred when deleting the products."""



from nisystemlink.clients.core._uplink._with_paging import WithPaging

from ._data_frame import DataFrame


class PagedTableRows(WithPaging):
    """Contains the result of a query for rows of data."""

    frame: DataFrame
    """The data frame containing the rows of data."""

    total_row_count: int
    """The total number of rows matched by the query across all pages of results."""



"""Implementation of TestMonitor Client"""

from typing import List, Optional

from nisystemlink.clients import core
from nisystemlink.clients.core._uplink._base_client import BaseClient
from nisystemlink.clients.core._uplink._methods import delete, get, post
from nisystemlink.clients.testmonitor.models import (
    CreateResultRequest,
    UpdateResultRequest,
)
from uplink import Field, Path, Query, retry, returns

from . import models


@retry(
    when=retry.when.status(408, 429, 502, 503, 504),
    stop=retry.stop.after_attempt(5),
    on_exception=retry.CONNECTION_ERROR,
)
class TestMonitorClient(BaseClient):
    # prevent pytest from thinking this is a test class
    __test__ = False

    def __init__(self, configuration: Optional[core.HttpConfiguration] = None):
        """Initialize an instance.

        Args:
            configuration: Defines the web server to connect to and information about
                how to connect. If not provided, the
                :class:`HttpConfigurationManager <nisystemlink.clients.core.HttpConfigurationManager>`
                is used to obtain the configuration.

        Raises:
            ApiException: if unable to communicate with the TestMonitor Service.
        """
        if configuration is None:
            configuration = core.HttpConfigurationManager.get_configuration()
        super().__init__(configuration, base_path="/nitestmonitor/v2/")

    @get("")
    def api_info(self) -> models.ApiInfo:
        """Get information about the available API operations.

        Returns:
            Information about available API operations.

        Raises:
            ApiException: if unable to communicate with the `ni``/nitestmonitor``` service.
        """
        ...

    @post("results", args=[Field("results")])
    def create_results(
        self, results: List[CreateResultRequest]
    ) -> models.CreateResultsPartialSuccess:
        """Creates one or more results and returns errors for failed creations.

        Args:
            results: A list of results to attempt to create.

        Returns: A list of created results, results that failed to create, and errors for
            failures.

        Raises:
            ApiException: if unable to communicate with the ``/nitestmonitor`` service of provided invalid
                arguments.
        """
        ...

    @get(
        "results",
        args=[Query("continuationToken"), Query("take"), Query("returnCount")],
    )
    def get_results(
        self,
        continuation_token: Optional[str] = None,
        take: Optional[int] = None,
        return_count: Optional[bool] = None,
    ) -> models.PagedResults:
        """Reads a list of results.

        Args:
            continuation_token: The token used to paginate results.
            take: The number of results to get in this request.
            return_count: Whether or not to return the total number of results available.

        Returns:
            A list of results.

        Raises:
            ApiException: if unable to communicate with the ``/nitestmonitor`` Service
                or provided an invalid argument.
        """
        ...

    @get("results/{id}")
    def get_result(self, id: str) -> models.Result:
        """Retrieves a single result by id.

        Args:
            id (str): Unique ID of a result.

        Returns:
            The single result matching `id`

        Raises:
            ApiException: if unable to communicate with the ``/nitestmonitor`` Service
                or provided an invalid argument.
        """
        ...

    @post("query-results")
    def query_results(self, query: models.QueryResultsRequest) -> models.PagedResults:
        """Queries for results that match the filter.

        Args:
            query : The query contains a DynamicLINQ query string in addition to other details
                about how to filter and return the list of results.

        Returns:
            A paged list of results with a continuation token to get the next page.

        Raises:
            ApiException: if unable to communicate with the ``/nitestmonitor`` Service or provided invalid
                arguments.
        """
        ...

    @returns.json  # type: ignore
    @post("query-result-values")
    def query_result_values(self, query: models.QueryResultValuesRequest) -> List[str]:
        """Queries for results that match the query and returns a list of the requested field.

        Args:
            query : The query for the fields.

        Returns:
            A list of the values of the queried field.

        Raises:
            ApiException: if unable to communicate with the ``/nitestmonitor`` Service or provided
                invalid arguments.
        """
        ...

    @post("update-results", args=[Field("results"), Field("replace")])
    def update_results(
        self, results: List[UpdateResultRequest], replace: bool = False
    ) -> models.UpdateResultsPartialSuccess:
        """Updates a list of results with optional field replacement.

        Args:
            `results`: A list of results to update. Results are matched for update by id.
            `replace`: Replace the existing fields instead of merging them. Defaults to `False`.
                If this is `True`, then `keywords` and `properties` for the result will be
                    replaced by what is in the `results` provided in this request.
                If this is `False`, then the `keywords` and `properties` in this request will
                    merge with what is already present in the server resource.

        Returns: A list of updates results, results that failed to update, and errors for
            failures.

        Raises:
            ApiException: if unable to communicate with the ``/nitestmonitor`` Service
                or provided an invalid argument.
        """
        ...

    @delete("results/{id}")
    def delete_result(self, id: str) -> None:
        """Deletes a single result by id.

        Args:
            id (str): Unique ID of a result.

        Raises:
            ApiException: if unable to communicate with the ``/nitestmonitor`` Service
                or provided an invalid argument.
        """
        ...

    @post("delete-results", args=[Field("ids")])
    def delete_results(
        self, ids: List[str]
    ) -> Optional[models.DeleteResultsPartialSuccess]:
        """Deletes multiple results.

        Args:
            ids (List[str]): List of unique IDs of results.

        Returns:
            A partial success if any results failed to delete, or None if all
            results were deleted successfully.

        Raises:
            ApiException: if unable to communicate with the ``/nitestmonitor`` Service
                or provided an invalid argument.
        """
        ...

    @post(
        "steps",
        args=[Field("steps"), Field("updateResultTotalTime")],
    )
    def create_steps(
        self,
        steps: List[models.CreateStepRequest],
        update_result_total_time: bool = False,
    ) -> models.CreateStepsPartialSuccess:
        """Creates one or more steps.

        Args:
            steps: A list of steps to create.
            update_result_total_time: Determine test result total time from the step total times.
                Defaults to False.

        Returns:
            A list of steps that were successfully created and ones that failed to be created.

        Raises:
            ApiException: if unable to communicate with the `/nitestmonitor` service or if there are
            invalid arguments.
        """
        ...

    @post("delete-steps", args=[Field("steps")])
    def delete_steps(
        self, steps: List[models.StepIdResultIdPair]
    ) -> Optional[models.DeleteStepsPartialSuccess]:
        """Deletes one or more steps by global ID.

        Args:
            steps: A list of step IDs and result IDs. Note that these are the global IDs and not the
            `step_id` that is local to a product and workspace.

        Returns:
            None if all deletes succeed otherwise a list of which IDs failed and which succeeded.

        Raises:
            ApiException: if unable to communicate with the `/nitestmonitor` service or if there
            invalid arguments.
        """
        ...

    @delete(
        "results/{resultId}/steps/{stepId}",
        args=[Path("resultId"), Path("stepId"), Query("updateResultTotalTime")],
    )
    def delete_step(
        self,
        result_id: str,
        step_id: str,
        update_result_total_time: Optional[bool] = False,
    ) -> None:
        """Deletes a single step.

        Args:
            result_id: The resultId of the step to delete.
            step_id: The stepId of the step to delete.
            update_result_total_time: Determine test result total time from the step total times.
                Defaults to False.

        Returns:
            None

        Raises:
            ApiException: if unable to communicate with the `/nitestmonitor` service or if there are
            invalid arguments.
        """
        ...

    @post("query-steps")
    def query_steps(self, query: models.QueryStepsRequest) -> models.PagedSteps:
        """Queries for steps that match the filters.

        Args:
            query: The query contains a product ID as well as a filter for steps under that product.

        Returns:
            A list of steps that match the filter.

        Raises:
            ApiException: if unable to communicate with the `/nitestmonitor` service or if there are
            invalid arguments.
        """
        ...

    @post(
        "update-steps",
        args=[
            Field("steps"),
            Field("updateResultTotalTime"),
            Field("replaceKeywords"),
            Field("replaceProperties"),
        ],
    )
    def update_steps(
        self,
        steps: List[models.UpdateStepRequest],
        update_result_total_time: bool = False,
        replace_keywords: bool = False,
        replace_properties: bool = False,
    ) -> models.UpdateStepsPartialSuccess:
        """Updates one or more steps.

        Update requires that the version field matches the version being updated from.

        Args:
            steps: a list of steps that are to be updated. Must include the global ID and
            each step being updated must match the version currently on the server.
            update_result_total_time: Determine test result total time from the step total times.
                Defaults to False.
            replace_keywords: Replace with existing keywords instead of merging them.
                Defaults to False.
            replace_properties: Replace with existing properties instead of merging them.
                Defaults to False.

        Returns
            A list of steps that were successfully updated and a list of ones that were not along
            with error messages for updates that failed.

        Raises:
            ApiException: if unable to communicate with the `/nitestmonitor` service or if there are
            invalid arguments.
        """
        ...

    @get(
        "steps",
        args=[Query("continuationToken"), Query("take"), Query("returnCount")],
    )
    def get_steps(
        self,
        continuation_token: Optional[str] = None,
        take: Optional[int] = None,
        return_count: Optional[bool] = None,
    ) -> models.PagedSteps:
        """Reads a list of steps.

        Args:
            continuation_token: The token used to paginate steps.
            take: The number of steps to get in this request.
            return_count: Whether or not to return the total number of steps available.

        Returns:
            A list of steps.

        Raises:
            ApiException: if unable to communicate with the `/nitestmonitor` service or if there are
            invalid arguments..
        """
        ...

    @get("results/{resultId}/steps/{stepId}", args=[Path("resultId"), Path("stepId")])
    def get_step(self, result_id: str, step_id: str) -> models.Step:
        """Gets a single step.

        Args:
            result_id: The resultId of the step to get.
            step_id: The stepId of the step to get.

        Returns:
            The step.

        Raises:
            ApiException: if unable to communicate with the `/nitestmonitor` service or if there are
            invalid arguments.
        """
        ...

    @returns.json  # type: ignore
    @post("query-step-values")
    def query_step_values(self, query: models.QueryStepValuesRequest) -> List[str]:
        """Queries values for a step field.

        Args:
            query: The query parameters.

        Returns:
            A list of values for the specified step field.

        Raises:
            ApiException: if unable to communicate with the `/nitestmonitor` service or if there are
            invalid arguments.
        """
        ...



from nisystemlink.clients.core.helpers import IteratorFileLike
from requests.models import Response


def file_like_response_handler(response: Response) -> IteratorFileLike:
    """Response handler for File-Like content."""
    return IteratorFileLike(response.iter_content(chunk_size=4096))



from ._api_info import Operation, V2Operations, ApiInfo
from ._named_value import NamedValue
from ._result import Result
from ._status import StatusType, Status
from ._step import Step
from ._step_data import Measurement, StepData
from ._paged_results import PagedResults
from ._paged_steps import PagedSteps
from ._delete_results_partial_success import DeleteResultsPartialSuccess
from ._delete_steps_partial_success import DeleteStepsPartialSuccess, StepIdResultIdPair
from ._create_results_partial_success import CreateResultsPartialSuccess
from ._create_steps_partial_success import CreateStepsPartialSuccess
from ._update_results_partial_success import UpdateResultsPartialSuccess
from ._update_steps_partial_success import UpdateStepsPartialSuccess
from ._create_result_request import CreateResultRequest
from ._create_steps_request import CreateStepRequest
from ._update_result_request import UpdateResultRequest
from ._update_steps_request import UpdateStepRequest
from ._query_results_request import (
    QueryResultsRequest,
    QueryResultValuesRequest,
    ResultField,
    ResultProjection,
)
from ._query_steps_request import (
    QueryStepsRequest,
    QueryStepValuesRequest,
    StepOrderBy,
    StepField,
    StepProjection,
)

# flake8: noqa



from typing import List, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel


class RemoveSystemsResponse(JsonModel):
    """Model for remove systems response containing the IDs of the systems which were deleted."""

    jid: Optional[str] = None
    """The job ID of the remove systems operation."""

    removed_ids: Optional[List[str]] = None
    """The IDs of the systems that were successfully removed."""

    failed_ids: Optional[List[str]] = None
    """The IDs of the systems that could not be removed."""



# -*- coding: utf-8 -*-

"""Implementation of HttpTagSelection."""

import datetime
from typing import Any, Awaitable, Callable, List, Optional, Sequence, Tuple, TypeVar

from nisystemlink.clients import core, tag as tbase
from nisystemlink.clients.core._internal._http_client import HttpClient, HttpResponse
from nisystemlink.clients.core._internal._timestamp_utilities import TimestampUtilities
from nisystemlink.clients.tag._core._manual_reset_timer import ManualResetTimer
from nisystemlink.clients.tag._core._serialized_tag_with_aggregates import (
    SerializedTagWithAggregates,
)
from nisystemlink.clients.tag._http._http_tag_subscription import HttpTagSubscription
from typing_extensions import final


T = TypeVar("T")


@final
class HttpTagSelection(tbase.TagSelection):
    def __init_subclass__(cls) -> None:
        raise TypeError("type 'HttpTagSelection' is not an acceptable base type")

    def __init__(
        self,
        client: HttpClient,
        tags: Sequence[tbase.TagData],
        *,
        _paths: Optional[Sequence[str]] = None
    ) -> None:
        """Initialize a selection using existing data.

        Args:
            client: The HTTP client object for communicating with the server.
            tags: The tags to store in the selection.

        Raises:
            ValueError: if ``tags`` contains tags that are None or have invalid paths.
            ValueError: if ``tags`` contains duplicate tags.
            ValueError: if ``tags`` is None.
        """
        super().__init__(tags, _paths)
        self._client = client
        self._api = client.at_uri("/nitag/v2/selections")
        self._selection_stale = False
        self._token = None  # Optional[str]

    @classmethod
    def open(cls, client: HttpClient, paths: Sequence[str]) -> "HttpTagSelection":
        """Initialize a selection using queried data.

        Args:
            client: The HTTP client object for communicating with the server.
            paths: The paths used in the query.

        Returns:
            The created selection.

        Raises:
            ValueError: if ``paths`` is None.
            ValueError: if ``paths`` contains duplicate paths.
            ApiException: if the API call fails.
        """
        api = client.at_uri("/nitag/v2/selections")

        selection, http_response = api.post("", data={"searchPaths": list(paths)})

        if selection is None or selection.get("id") is None:
            raise tbase.TagManager.invalid_response(http_response)

        token = selection["id"]

        try:
            tags, http_response = api.get("/{id}/tags", params={"id": token})

            if tags is None:
                raise tbase.TagManager.invalid_response(http_response)

            selection = HttpTagSelection(
                client, [tbase.TagData.from_json_dict(t) for t in tags], _paths=paths
            )
            selection._token = token
            return selection
        except core.ApiException:
            try:
                api.delete("/{id}", params={"id": token})
            except core.ApiException:
                pass

            raise

    @classmethod
    async def open_async(
        cls, client: HttpClient, paths: Sequence[str]
    ) -> "HttpTagSelection":
        """Asynchronously initialize a selection using queried data.

        Args:
            client: The HTTP client object for communicating with the server.
            paths: The paths used in the query.

        Returns:
            A task representing the asynchronous operation. On completion, contains the
            created selection.

        Raises:
            ValueError: if ``paths`` is None.
            ValueError: if ``paths`` contains duplicate paths.
            ApiException: if the API call fails.
        """
        api = client.at_uri("/nitag/v2/selections").as_async

        selection, http_response = await api.post("", data={"searchPaths": list(paths)})

        if selection is None or selection.get("id") is None:
            raise tbase.TagManager.invalid_response(http_response)

        token = selection["id"]

        try:
            tags, http_response = await api.get("/{id}/tags", params={"id": token})

            if tags is None:
                raise tbase.TagManager.invalid_response(http_response)

            selection = HttpTagSelection(
                client, [tbase.TagData.from_json_dict(t) for t in tags], _paths=paths
            )
            selection._token = token
            return selection
        except core.ApiException:
            try:
                await api.delete("/{id}", params={"id": token})
            except core.ApiException:
                pass

            raise

    def _on_paths_changed(self) -> None:
        self._selection_stale = True

    def _close_internal(self) -> None:
        if self._token is None:
            return

        try:
            self._api.delete("/{id}", params={"id": self._token})
        except core.ApiException:
            pass

    async def _close_internal_async(self) -> None:
        if self._token is None:
            return

        try:
            await self._api.as_async.delete("/{id}", params={"id": self._token})
        except core.ApiException:
            pass

    def _create_subscription_internal(
        self, update_interval: Optional[datetime.timedelta] = None
    ) -> tbase.TagSubscription:
        update_timer = None  # type: Optional[ManualResetTimer]
        if update_interval is not None:
            update_timer = ManualResetTimer(update_interval)
        paths = set(self.paths).union(self.metadata.keys())
        return HttpTagSubscription.create(
            self._client, paths, update_timer, heartbeat_timer=None
        )

    async def _create_subscription_internal_async(
        self, update_interval: Optional[datetime.timedelta] = None
    ) -> tbase.TagSubscription:
        update_timer = None  # type: Optional[ManualResetTimer]
        if update_interval is not None:
            update_timer = ManualResetTimer(update_interval)
        paths = set(self.paths).union(self.metadata.keys())
        return await HttpTagSubscription.create_async(
            self._client, paths, update_timer, heartbeat_timer=None
        )

    def _delete_tags_from_server_internal(self) -> None:
        def fn(token: str) -> None:
            self._api.delete("/{id}/tags", params={"id": token})

        self._ensure_selection_and_call(fn)

    async def _delete_tags_from_server_internal_async(self) -> None:
        def fn(token: str) -> Awaitable[Tuple[None, HttpResponse]]:
            return self._api.as_async.delete("/{id}/tags", params={"id": token})

        await self._ensure_selection_and_call_async(fn)

    def _read_tag_metadata(self) -> List[tbase.TagData]:
        def fn(token: str) -> Tuple[Any, HttpResponse]:
            return self._api.get("/{id}/tags", params={"id": token})

        response, http_response = self._ensure_selection_and_call(fn)
        return self._handle_read_tags_metadata(response, http_response)

    async def _read_tag_metadata_async(self) -> List[tbase.TagData]:
        def fn(token: str) -> Awaitable[Tuple[Any, HttpResponse]]:
            return self._api.as_async.get("/{id}/tags", params={"id": token})

        response, http_response = await self._ensure_selection_and_call_async(fn)
        return self._handle_read_tags_metadata(response, http_response)

    def _handle_read_tags_metadata(
        self, response: List[Any], http_response: HttpResponse
    ) -> List[tbase.TagData]:
        if response is None or any(t is None for t in response):
            raise tbase.TagManager.invalid_response(http_response)

        return [tbase.TagData.from_json_dict(t) for t in response]

    def _read_tag_values(self) -> List[Optional[SerializedTagWithAggregates]]:
        def fn(token: str) -> Tuple[Any, HttpResponse]:
            return self._api.get("/{id}/values", params={"id": token})

        response, http_response = self._ensure_selection_and_call(fn)
        return self._handle_read_tags_values(response, http_response)

    async def _read_tag_values_async(
        self,
    ) -> List[Optional[SerializedTagWithAggregates]]:
        def fn(token: str) -> Awaitable[Tuple[Any, HttpResponse]]:
            return self._api.as_async.get("/{id}/values", params={"id": token})

        response, http_response = await self._ensure_selection_and_call_async(fn)
        return self._handle_read_tags_values(response, http_response)

    def _handle_read_tags_values(
        self,
        response: List[Any],
        http_response: HttpResponse,
        paths: Optional[List[str]] = None,
    ) -> List[Optional[SerializedTagWithAggregates]]:
        if response is None or any(t is None for t in response):
            raise tbase.TagManager.invalid_response(http_response)

        result = []  # type: List[Optional[SerializedTagWithAggregates]]
        for i, t in enumerate(response):
            path = paths[i] if paths else t.get("path")
            if path is None:
                raise tbase.TagManager.invalid_response(http_response)

            if not t.get("current"):
                result.append(None)
                continue

            v = t["current"].get("value", {})
            value = v.get("value")
            data_type = v.get("type")
            aggregates = t.get("aggregates") or {}
            timestamp = None  # type: Optional[datetime.datetime]
            if t["current"].get("timestamp"):
                timestamp = TimestampUtilities.str_to_datetime(
                    t["current"]["timestamp"]
                )
            if value is None or data_type is None:
                raise tbase.TagManager.invalid_response(http_response)

            result.append(
                SerializedTagWithAggregates(
                    path,
                    tbase.DataType.from_api_name(data_type),
                    value,
                    timestamp,
                    aggregates.get("count"),
                    aggregates.get("min"),
                    aggregates.get("max"),
                    (
                        float(aggregates["avg"])
                        if aggregates.get("avg") is not None
                        else None
                    ),
                )
            )
        return result

    def _read_tag_metadata_and_values(
        self,
    ) -> Tuple[List[tbase.TagData], List[Optional[SerializedTagWithAggregates]]]:
        def fn(token: str) -> Tuple[Any, HttpResponse]:
            return self._api.get("/{id}/tags-with-values", params={"id": token})

        response, http_response = self._ensure_selection_and_call(fn)
        return self._handle_read_tags_metadata_and_values(response, http_response)

    async def _read_tag_metadata_and_values_async(
        self,
    ) -> Tuple[List[tbase.TagData], List[Optional[SerializedTagWithAggregates]]]:
        def fn(token: str) -> Awaitable[Tuple[Any, HttpResponse]]:
            return self._api.as_async.get(
                "/{id}/tags-with-values", params={"id": token}
            )

        response, http_response = await self._ensure_selection_and_call_async(fn)
        return self._handle_read_tags_metadata_and_values(response, http_response)

    def _handle_read_tags_metadata_and_values(
        self, response: Any, http_response: HttpResponse
    ) -> Tuple[List[tbase.TagData], List[Optional[SerializedTagWithAggregates]]]:
        if (
            response is None
            or response.get("tagsWithValues") is None
            or any(
                t is None or t.get("tag") is None for t in response["tagsWithValues"]
            )
        ):
            raise tbase.TagManager.invalid_response(http_response)

        return (
            self._handle_read_tags_metadata(
                [t["tag"] for t in response["tagsWithValues"]], http_response
            ),
            self._handle_read_tags_values(
                response["tagsWithValues"],
                http_response,
                [t["tag"]["path"] for t in response["tagsWithValues"]],
            ),
        )

    def _reset_aggregates_internal(self) -> None:
        def fn(token: str) -> None:
            self._api.post("/{id}/reset-aggregates", params={"id": token})

        self._ensure_selection_and_call(fn)

    async def _reset_aggregates_internal_async(self) -> None:
        def fn(token: str) -> Awaitable[Tuple[None, HttpResponse]]:
            return self._api.as_async.post(
                "/{id}/reset-aggregates", params={"id": token}
            )

        await self._ensure_selection_and_call_async(fn)

    def _create_selection_on_server(self) -> None:
        selection, http_response = self._api.post(
            "", data={"searchPaths": list(self.paths)}
        )

        if selection is None or selection.get("id") is None:
            raise tbase.TagManager.invalid_response(http_response)

        self._token = selection["id"]
        self._selection_stale = False

    async def _create_selection_on_server_async(self) -> None:
        selection, http_response = await self._api.as_async.post(
            "", data={"searchPaths": list(self.paths)}
        )

        if selection is None or selection.get("id") is None:
            raise tbase.TagManager.invalid_response(http_response)

        self._token = selection["id"]
        self._selection_stale = False

    def _update_selection_on_server_if_needed(self) -> None:
        if self._selection_stale:
            self._api.put(
                "/{id}",
                data={"searchPaths": list(self.paths), "id": self._token},
                params={"id": self._token},
            )
            self._selection_stale = False

    async def _update_selection_on_server_if_needed_async(self) -> None:
        if self._selection_stale:
            await self._api.as_async.put(
                "/{id}",
                data={"searchPaths": list(self.paths), "id": self._token},
                params={"id": self._token},
            )
            self._selection_stale = False

    def _ensure_selection_and_call(self, api_call: Callable[[str], T]) -> T:
        if self._token is not None:
            try:
                self._update_selection_on_server_if_needed()
                return api_call(self._token)
            except core.ApiException as ex:
                if ex.http_status_code == 404:
                    # The server must have deleted it for inactivity. Recreate it below.
                    self._token = None
                else:
                    raise

        self._create_selection_on_server()
        assert self._token is not None
        return api_call(self._token)

    async def _ensure_selection_and_call_async(
        self, api_call: Callable[[str], Awaitable[T]]
    ) -> T:
        if self._token is not None:
            try:
                await self._update_selection_on_server_if_needed_async()
                return await api_call(self._token)
            except core.ApiException as ex:
                if ex.http_status_code == 404:
                    # The server must have deleted it for inactivity. Recreate it below.
                    self._token = None
                else:
                    raise

        await self._create_selection_on_server_async()
        assert self._token is not None
        return await api_call(self._token)



# -*- coding: utf-8 -*-

"""Implementation of ManualResetTimer."""

import datetime
import threading
import traceback
from types import TracebackType
from typing import Any, Callable, List, Optional, Type

import events
from nisystemlink.clients.core._internal._classproperty_support import (
    ClasspropertySupport,
)
from typing_extensions import final, Literal


@final
class ManualResetTimer(events.Events, metaclass=ClasspropertySupport):
    """Represents a timer for periodic background operations such that :meth:`start()`
    must be called to restart the timer each time the :attr:`elapsed` event is raised.

    Attributes:
        elapsed: An event that is triggered when the timer has elapsed.

    Example::

        def timer_elapsed():
            print("The timer elapsed!")

        my_timer.elapsed += timer_elapsed
    """

    def __init_subclass__(cls) -> None:
        raise TypeError("type 'ManualResetTimer' is not an acceptable base type")

    __events__ = ["elapsed"]
    # Under certain circumstances, mypy complains about the event not having a type hint
    # unless we specify it explicitly. (But we also need to delete the attribute so that
    # Events.__getattr__ can do its magic.)
    elapsed = None  # type: events._EventSlot
    del elapsed

    __null_timer_impl = None  # type: Optional[ManualResetTimer]

    @ClasspropertySupport.classproperty
    def null_timer(cls) -> "ManualResetTimer":
        """A timer that never fires."""
        if cls.__null_timer_impl is None:
            # Bypass the ManualResetTimer constructor, to make a timerless timer
            obj = cls.__new__(ManualResetTimer)
            super(ManualResetTimer, obj).__init__()  # but call the base constructor

            obj._thread = None
            obj._running = []  # used as mutable boolean; empty is False

            cls.__null_timer_impl = obj
        return cls.__null_timer_impl

    def __init__(self, interval: datetime.timedelta) -> None:
        """Initialize a timer that fires at the given interval a single time once
        :meth:`start()` has been called and then automatically stops.

        Args:
            interval: The amount of time after calling :meth:`start()` before
                :attr:`elapsed` is raised.

        Raises:
            ValueError: if ``interval`` is less than or equal to zero.
        """
        super().__init__()
        interval_secs = interval.total_seconds()
        if interval_secs <= 0:
            raise ValueError("interval cannot be <= 0")

        # Note: This _running flag means that the *thread* is running, not the timer
        self._running = [None]  # used as mutable boolean; non-empty is True
        self._timer_start = threading.Event()
        self._timer_cancel = threading.Event()
        self._thread = threading.Thread(
            target=self._run,
            args=[
                self._running,
                interval_secs,
                self._timer_start,
                self._timer_cancel,
                self.elapsed,
            ],
        )  # type: Optional[threading.Thread]
        self._thread.daemon = True
        self._thread.start()

    @property
    def can_start(self) -> bool:  # noqa: D401
        """Whether or not the timer is configured and can be started.

        A timer that isn't configured will never raise :attr:`elapsed`, even when
        :meth:`start()` is called.
        """
        return self._thread is not None

    def start(self) -> None:
        """Start the timer."""
        if self._running:
            self._timer_cancel.clear()
            self._timer_start.set()

    def stop(self) -> None:
        """Stop the timer."""
        if self._running:
            self._timer_cancel.set()

    @staticmethod
    def _run(
        running: List[None],
        interval: int,
        timer_start: threading.Event,
        timer_cancel: threading.Event,
        elapsed: Callable[[], None],
    ) -> None:
        while running:
            timer_start.wait()
            timer_start.clear()
            if running and not timer_cancel.wait(interval):
                try:
                    elapsed()
                except Exception:
                    traceback.print_exc()

    def __enter__(self) -> "ManualResetTimer":
        return self

    async def __aenter__(self) -> "ManualResetTimer":
        return self

    def __exit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> Literal[False]:
        self.stop()
        for handler in list(self.elapsed):
            self.elapsed -= handler
        return False

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> Literal[False]:
        self.stop()
        for handler in list(self.elapsed):
            self.elapsed -= handler
        return False

    def __del__(self) -> None:
        self.stop()
        for handler in list(self.elapsed):
            self.elapsed -= handler

        # Stop the thread
        if self._running:
            self._running.clear()  # set our "mutable boolean" to False
            self._thread = None
            self._timer_cancel.set()
            self._timer_start.set()

    # Work around https://github.com/pyeve/events/issues/17
    def __getattr__(self, name: str) -> Any:
        if name in self.__events__:
            return super().__getattr__(name)
        else:
            return object.__getattribute__(self, name)

    # Fake method to tell mypy the type of our events
    def __type_hinting__(self) -> None:
        self.elapsed = type(self).elapsed  # type: events._EventSlot



from datetime import datetime
from typing import List, Optional

from nisystemlink.clients.core import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel
from nisystemlink.clients.spec.models._specification import (
    SpecificationDefinition,
    SpecificationType,
)


class CreateSpecificationsRequestObject(SpecificationDefinition):
    product_id: str
    """Id of the product to which the specification will be associated."""

    spec_id: str
    """User provided value using which the specification will be identified.

    This should be unique for a product and workspace combination.
    """

    type: SpecificationType
    """Type of the specification."""


class CreateSpecificationsRequest(JsonModel):
    """Create multiple specifications."""

    specs: Optional[List[CreateSpecificationsRequestObject]] = None
    """List of specifications to be created."""


class BaseSpecificationResponse(JsonModel):
    """Base Response Model for create specs response and update specs response."""

    id: str
    """The global Id of the specification."""

    product_id: str
    """Id of the product to which the specification will be associated."""

    spec_id: str
    """User provided value using which the specification will be identified.

    This should be unique for a product and workspace combination.
    """

    workspace: str
    """Id of the workspace to which the specification will be associated.

    Default workspace will be taken if the value is not given.
    """

    version: int
    """
    Current version of the specification.

    When an update is applied, the version is automatically incremented.
    """


class CreatedSpecification(BaseSpecificationResponse):
    """A specification successfully created on the server."""

    created_at: datetime
    """ISO-8601 formatted timestamp indicating when the specification was created."""

    created_by: str
    """Id of the user who created the specification."""


class CreateSpecificationsPartialSuccess(JsonModel):
    """When some specs can not be created, this contains the list that was and was not created."""

    created_specs: Optional[List[CreatedSpecification]] = None
    """Information about the created specification(s)"""

    failed_specs: Optional[List[CreateSpecificationsRequestObject]] = None
    """List of specification requests that failed during creation."""

    error: Optional[ApiError] = None



from typing import List, Optional

from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel


class LinkFilesPartialSuccessResponse(JsonModel):
    """Model for a Link Files Partial Success Response."""

    succeeded: Optional[List[str]] = None
    """Gets or sets array of file IDs that were successfully linked to assets."""

    failed: Optional[List[str]] = None
    """Gets or sets array of file IDs that failed to link to assets."""

    error: Optional[ApiError] = None
    """Gets or sets error if the link file operation failed."""



"""Implementation of SystemsClient"""

from typing import List, Optional

from nisystemlink.clients import core
from nisystemlink.clients.core._uplink._base_client import BaseClient
from nisystemlink.clients.core._uplink._methods import post
from uplink import Field, retry

from . import models


@retry(
    when=retry.when.status(408, 429, 502, 503, 504),
    stop=retry.stop.after_attempt(5),
    on_exception=retry.CONNECTION_ERROR,
)
class SystemsClient(BaseClient):

    def __init__(self, configuration: Optional[core.HttpConfiguration] = None):
        """Initialize an instance.

        Args:
            configuration: Defines the web server to connect to and information about
                how to connect. If not provided, the
                :class:`HttpConfigurationManager <nisystemlink.clients.core.HttpConfigurationManager>`
                is used to obtain the configuration.

        Raises:
            ApiException: if unable to communicate with the Systems Service.
        """
        if configuration is None:
            configuration = core.HttpConfigurationManager.get_configuration()

        super().__init__(configuration, base_path="/nisysmgmt/v1/")

    @post("virtual")
    def create_virtual_system(
        self, create_virtual_system_request: models.CreateVirtualSystemRequest
    ) -> models.CreateVirtualSystemResponse:
        """Creates a virtual system.

        Args:
            alias: The alias of the virtual system.
            workspace: The workspace to create the virtual system in.

        Raises:
            ApiException: if unable to communicate with the `/nisysmgmt` service or provided invalid
                arguments.
        """
        ...

    @post("query-systems")
    def query_systems(
        self, query: models.QuerySystemsRequest
    ) -> models.QuerySystemsResponse:
        """Queries for systems that match the filter.

        Args:
            query : The query contains a DynamicLINQ query string in addition to other details
                about how to filter and return the list of systems.

        Returns:
            Response containing the list of systems that match the filter.

        Raises:
            ApiException: if unable to communicate with the `/nisysmgmt` Service or provided invalid
                arguments.
        """
        ...

    @post("remove-systems", args=[Field("tgt")])
    def remove_systems(self, tgt: List[str]) -> models.RemoveSystemsResponse:
        """Removes multiple systems.

        Args:
            virtual_system_to_remove : List of unique IDs of systems.

        Returns:
            A partial success if any systems failed to remove, or None if all
            systems were removed successfully.

        Raises:
            ApiException: if unable to communicate with the `/nisysmgmt` Service
                or provided an invalid argument.
        """
        ...



from typing import Any, Dict, List, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel


class QuerySystemsResponse(JsonModel):
    """Model for query systems request."""

    count: Optional[int] = None
    """Number of systems match the query."""

    data: Optional[List[Dict[str, Any]]] = None
    """Contains info of the queried systems."""



from __future__ import annotations

from datetime import datetime
from typing import List, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._platform import Platform


class Feed(JsonModel):
    """Feed model."""

    id: Optional[str] = None
    """The auto-generated ID of the feed."""
    name: Optional[str] = None
    """The name of the feed."""
    description: Optional[str] = None
    """The description of the feed."""
    platform: Optional[Platform] = None
    """The platform of the feed, the following package extensions are available: .nipkg for
    windows feeds, .ipk and .deb for ni-linux-rt feeds.
    """
    workspace: Optional[str] = None
    """The ID of the workspace this feed belongs to."""
    updated_at: Optional[datetime] = None
    """The date of the latest feed update"""
    created_at: Optional[datetime] = None
    """The date when the feed was created at."""
    package_sources: Optional[List[str]] = None
    """The package sources list of the feed."""
    deleted: Optional[bool] = None
    """Whether the feed deletion was requested."""



from typing import Dict, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel


class Dashboard(JsonModel):
    """Represents a dashboard reference."""

    id: Optional[str] = None
    """ID of the dashboard"""

    variables: Optional[Dict[str, str]] = None
    """Variables for the dashboard"""


class DashboardUrl(Dashboard):
    """Definition and URL of the dashboard reference associated with this test plan."""

    url: Optional[str] = None
    """URL of the dashboard reference associated with this test plan."""



from typing import Dict, List, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel


class BaseProductRequest(JsonModel):
    """Contains information about a product."""

    name: Optional[str] = None
    """The name of the product.

    Usually the name is used to refer to several part numbers that all have the same name but
    different revisions or versions.
    """

    family: Optional[str] = None
    """The family that that this product belongs to.

    Usually the family is a grouping above product name. A family usually has multiple product
    names within it.
    """

    file_ids: Optional[List[str]] = None
    """A list of file ids that are attached to this product."""

    keywords: Optional[List[str]] = None
    """A list of keywords that categorize this product."""

    properties: Optional[Dict[str, str]] = None
    """A list of custom properties for this product."""

    workspace: Optional[str] = None
    """The id of the workspace that this product belongs to."""


class CreateProductRequest(BaseProductRequest):

    part_number: str
    """The part number is the unique identifier of a product within a single org.

    Usually the part number refers to a specific revision or version of a given product."""


class UpdateProductRequest(BaseProductRequest):
    """This is the request model to update a product."""

    id: str
    """The globally unique id of the product."""



from typing import List, Optional

from nisystemlink.clients.core import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel


class StepIdResultIdPair(JsonModel):
    step_id: str
    """The ID of the step."""

    result_id: str
    """The ID of the result associated with the step."""


class DeleteStepsPartialSuccess(JsonModel):
    """The result of deleting multiple steps when one or more steps could not be deleted."""

    steps: List[StepIdResultIdPair]
    """The step_id and result_id pairs of the steps that were successfully deleted."""

    failed: Optional[List[StepIdResultIdPair]] = None
    """The step_id and result_id pairs of the steps that could not be deleted."""

    error: Optional[ApiError] = None
    """The error that occurred when deleting the steps."""



from datetime import datetime
from typing import List, Optional

from nisystemlink.clients.core import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel
from nisystemlink.clients.spec.models._create_specs_request import (
    BaseSpecificationResponse,
)
from nisystemlink.clients.spec.models._specification import (
    SpecificationDefinition,
    SpecificationType,
)


class UpdateSpecificationsRequestObject(SpecificationDefinition):
    id: str
    """The global Id of the specification."""

    product_id: str
    """Id of the product to which the specification will be associated."""

    spec_id: str
    """User provided value using which the specification will be identified.

    This should be unique for a product and workspace combination.
    """

    type: SpecificationType
    """Type of the specification."""

    workspace: str
    """Id of the workspace to which the specification will be associated.

    Default workspace will be taken if the value is not given.
    """

    version: int
    """
    Current version of the specification.

    When an update is applied, the version is automatically incremented.
    """


class UpdateSpecificationsRequest(JsonModel):

    specs: Optional[List[UpdateSpecificationsRequestObject]] = None
    """List of specifications to be updated."""


class UpdatedSpecification(BaseSpecificationResponse):
    """A specification that was updated on the server."""

    updated_at: datetime
    """ISO-8601 formatted timestamp indicating when the specification was last updated."""

    updated_by: str
    """Id of the user who last updated the specification."""


class UpdateSpecificationsPartialSuccess(JsonModel):

    updated_specs: Optional[List[UpdatedSpecification]] = None
    """Information about each of the updated specification(s)."""

    failed_specs: Optional[List[UpdateSpecificationsRequestObject]] = None
    """Information about each of the specification request(s) that failed during the update."""

    error: Optional[ApiError] = None
    """Any errors that occurred."""



from typing import List, Optional

from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._create_test_plan_request import CreateTestPlanRequest
from ._test_plan import TestPlan


class CreateTestPlansPartialSuccessResponse(JsonModel):
    """Represents the response from creating test plans, including successfully created,
    failed test plans, and any associated errors.
    """

    created_test_plans: Optional[List[TestPlan]] = None
    """List of test plans that were successfully created."""

    failed_test_plans: Optional[List[CreateTestPlanRequest]] = None
    """List of test plans that failed to be created, with their request body content."""

    error: Optional[ApiError] = None
    """The error that occurred when creating the test plans."""



from pydantic import BaseModel, ConfigDict


def _camelcase(s: str) -> str:
    """Convert a snake case string to camelCase."""
    parts = iter(s.split("_"))
    return next(parts) + "".join(i.title() for i in parts)


class JsonModel(BaseModel):
    """Base class for models that are serialized to and from JSON."""

    model_config = ConfigDict(
        alias_generator=_camelcase,
        validate_by_name=True,
        validate_by_alias=True,
        extra="ignore",
    )



"""Implementation of FileClient."""

import json
from typing import BinaryIO, Dict, List, Optional

from nisystemlink.clients import core
from nisystemlink.clients.core._uplink._base_client import BaseClient
from nisystemlink.clients.core._uplink._file_like_response import (
    file_like_response_handler,
)
from nisystemlink.clients.core._uplink._methods import (
    delete,
    get,
    post,
    response_handler,
)
from nisystemlink.clients.core.helpers import IteratorFileLike
from requests.models import Response
from uplink import Body, Field, params, Part, Path, Query, retry

from . import models


def _file_uri_response_handler(response: Response) -> str:
    """Response handler for File URI response. Extracts ID from URI."""
    resp = response.json()
    uri: str = resp["uri"]
    # Split the uri by '/' and get the last part
    parts = uri.split("/")
    return parts[-1]


@retry(when=retry.when.status(429), stop=retry.stop.after_attempt(5))
class FileClient(BaseClient):
    def __init__(self, configuration: Optional[core.HttpConfiguration] = None):
        """Initialize an instance.

        Args:
            configuration: Defines the web server to connect to and information about
                how to connect. If not provided, the
                :class:`HttpConfigurationManager <nisystemlink.clients.core.HttpConfigurationManager>`
                is used to obtain the configuration.

        Raises:
            ApiException: if unable to communicate with the File Service.
        """
        if configuration is None:
            configuration = core.HttpConfigurationManager.get_configuration()

        super().__init__(configuration, "/nifile/v1/")

    @get("")
    def api_info(self) -> models.V1Operations:
        """Get information about available API operations.

        Returns:
            Information about available API operations.

        Raises:
            ApiException: if unable to communicate with the File Service.
        """

    @get(
        "service-groups/Default/files",
        args=[
            Query,
            Query,
            Query(name="orderBy"),
            Query(name="orderByDescending"),
            Query(name="id"),
        ],
    )
    def __get_files(
        self,
        skip: int = 0,
        take: int = 0,
        order_by: Optional[str] = None,
        order_by_descending: Optional[str] = "false",
        ids: Optional[str] = None,
    ) -> models.FileQueryResponse:
        """Lists available files on the SystemLink File service.
        Use the skip and take parameters to return paged responses.
        The orderBy and orderByDescending fields can be used to manage sorting the list by metadata objects.

        Args:
            skip: How many files to skip in the result when paging. Defaults to 0.
            take: How many files to return in the result, or 0 to use a default defined by the service.
              Defaults to 0.
            order_by: The name of the metadata key to sort by. Defaults to None.
            order_by_descending: The elements in the list are sorted ascending if "false"
              and descending if "true". Defaults to "false".
            ids: Comma-separated list of file IDs to search by. Defaults to None.

        Returns:
            File Query Response

        Raises:
            ApiException: if unable to communicate with the File Service.
        """

    def get_files(
        self,
        skip: int = 0,
        take: int = 0,
        order_by: Optional[models.FileQueryOrderBy] = None,
        order_by_descending: Optional[bool] = False,
        ids: Optional[List[str]] = None,
    ) -> models.FileQueryResponse:
        """Lists available files on the SystemLink File service.
        Use the skip and take parameters to return paged responses.
        The orderBy and orderByDescending fields can be used to manage sorting the list by metadata objects.

        Args:
            skip: How many files to skip in the result when paging. Defaults to 0.
            take: How many files to return in the result, or 0 to use a default defined by the service.
            Defaults to 0.
            order_by: The name of the metadata key to sort by. Defaults to None.
            order_by_descending: The elements in the list are sorted ascending if False
            and descending if True. Defaults to False.
            ids: List of file IDs to search by. Defaults to None.

        Returns:
            File Query Response

        Raises:
            ApiException: if unable to communicate with the File Service.
        """
        # Uplink does not support enum serializing into str
        # workaround as the service expects lower case `true` and `false`
        # uplink serializes bools to `True` and `False`
        order_by_str = order_by.value if order_by is not None else None
        order_by_desc_str = "true" if order_by_descending else "false"

        if ids:
            ids_str = ",".join(ids)
        else:
            ids_str = ""

        resp = self.__get_files(
            skip=skip,
            take=take,
            order_by=order_by_str,
            order_by_descending=order_by_desc_str,
            ids=ids_str,
        )

        return resp

    @params({"force": True})  # type: ignore
    @delete("service-groups/Default/files/{id}", args=[Path])
    def delete_file(self, id: str) -> None:
        """Deletes the file indicated by the `file_id`.

        Args:
            id: The ID of the file.

        Raises:
            ApiException: if unable to communicate with the File Service.
        """

    @params({"force": True})  # type: ignore
    @post("service-groups/Default/delete-files", args=[Field])
    def delete_files(self, ids: List[str]) -> None:
        """Delete multiple files.

        Args:
            ids: List of unique IDs of Files.

        Raises:
            ApiException: if unable to communicate with the File Service.
        """

    @params({"inline": True})  # type: ignore
    @response_handler(file_like_response_handler)
    @get("service-groups/Default/files/{id}/data", args=[Path])
    def download_file(self, id: str) -> IteratorFileLike:
        """Downloads a file from the SystemLink File service.

        Args:
            id: The ID of the file.

        Yields:
            A file-like object for reading the exported data.

        Raises:
            ApiException: if unable to communicate with the File Service.
        """

    @response_handler(_file_uri_response_handler)
    @post("service-groups/Default/upload-files")
    def __upload_file(
        self,
        file: Part,
        metadata: Part = None,
        id: Part = None,
        workspace: Query = None,
    ) -> str:
        """Uploads a file using multipart/form-data headers to send the file payload in the HTTP body.

        Args:
            file: The file to upload.
            metadata: JSON Dictionary with key/value pairs
            id: Specify an unique (among all file) 24-digit Hex string ID of the file once it is uploaded.
                Defaults to None.
            workspace: The id of the workspace the file belongs to. Defaults to None.

        Returns:
            ID of uploaded file.

        Raises:
            ApiException: if unable to communicate with the File Service.
        """

    def upload_file(
        self,
        file: BinaryIO,
        metadata: Optional[Dict[str, str]] = None,
        id: Optional[str] = None,
        workspace: Optional[str] = None,
    ) -> str:
        """Uploads a file to the File Service.

        Args:
            file: The file to upload.
            metadata: File Metadata as dictionary.
            id: Specify an unique (among all file) 24-digit Hex string ID of the file once it is uploaded.
                Defaults to None.
            workspace: The id of the workspace the file belongs to. Defaults to None.

        Returns:
            ID of uploaded file.

        Raises:
            ApiException: if unable to communicate with the File Service.
        """
        if metadata:
            metadata_str = json.dumps(metadata)
        else:
            metadata_str = None

        file_id = self.__upload_file(
            file=file,
            metadata=metadata_str,
            id=id,
            workspace=workspace,
        )

        return file_id

    @post("service-groups/Default/files/{id}/update-metadata", args=[Body, Path])
    def update_metadata(self, metadata: models.UpdateMetadataRequest, id: str) -> None:
        """Updates an existing file's metadata with the specified metadata properties.

        Args:
            metadata: File's metadata and options for updating it.
            id: ID of the file to update Metadata.

        Raises:
            ApiException: if unable to communicate with the File Service.
        """



from enum import Enum
from typing import List, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel
from nisystemlink.clients.core._uplink._with_paging import WithPaging
from pydantic import Field


class ResultField(str, Enum):
    """The allowed field for which the values can be queried for."""

    ID = "ID"
    STARTED_AT = "STARTED_AT"
    UPDATED_AT = "UPDATED_AT"
    PROGRAM_NAME = "PROGRAM_NAME"
    SYSTEM_ID = "SYSTEM_ID"
    HOST_NAME = "HOST_NAME"
    OPERATOR = "OPERATOR"
    SERIAL_NUMBER = "SERIAL_NUMBER"
    PART_NUMBER = "PART_NUMBER"
    PRODUCT = "PRODUCT"
    TOTAL_TIME_IN_SECONDS = "TOTAL_TIME_IN_SECONDS"


class ResultOrderByField(str, Enum):
    """The fields by which results can be ordered."""

    ID = "ID"
    STARTED_AT = "STARTED_AT"
    UPDATED_AT = "UPDATED_AT"
    PROGRAM_NAME = "PROGRAM_NAME"
    SYSTEM_ID = "SYSTEM_ID"
    HOST_NAME = "HOST_NAME"
    OPERATOR = "OPERATOR"
    SERIAL_NUMBER = "SERIAL_NUMBER"
    PART_NUMBER = "PART_NUMBER"
    PRODUCT = "PRODUCT"
    TOTAL_TIME_IN_SECONDS = "TOTAL_TIME_IN_SECONDS"
    PROPERTIES = "PROPERTIES"


class ComparisonType(str, Enum):
    """The valid ways to order a result query."""

    DEFAULT = "DEFAULT"
    NUMERIC = "NUMERIC"
    LEXICOGRAPHIC = "LEXICOGRAPHIC"


class ResultProjection(str, Enum):
    """The allowed projections for query.

    When using projection, only the fields specified by the projection element will be included in
    the response.
    """

    ID = "ID"
    STATUS = "STATUS"
    STARTED_AT = "STARTED_AT"
    UPDATED_AT = "UPDATED_AT"
    PROGRAM_NAME = "PROGRAM_NAME"
    SYSTEM_ID = "SYSTEM_ID"
    HOST_NAME = "HOST_NAME"
    OPERATOR = "OPERATOR"
    SERIAL_NUMBER = "SERIAL_NUMBER"
    PART_NUMBER = "PART_NUMBER"
    TOTAL_TIME_IN_SECONDS = "TOTAL_TIME_IN_SECONDS"
    KEYWORDS = "KEYWORDS"
    PROPERTIES = "PROPERTIES"
    FILE_IDS = "FILE_IDS"
    DATA_TABLE_IDS = "DATA_TABLE_IDS"
    STATUS_TYPE_SUMMARY = "STATUS_TYPE_SUMMARY"
    WORKSPACE = "WORKSPACE"


class QueryResultsBase(JsonModel):
    filter: Optional[str] = None
    """
    The result query filter in Dynamic Linq format.
    Allowed properties in the filter are:
    - `id`: String for the global identifier of the result
    - `status.statusType`: String enumeration representing the status of the result.
        Possible values are : LOOPING, SKIPPED, CUSTOM, DONE, PASSED, FAILED,
        RUNNING, WAITING, TERMINATED, ERRORED, TIMED_OUT
    - `systemId`: String for the system identifier of the result
    - `hostName`: String for the host name of the result
    - `operator`: String for the operator of the result
    - `serialNumber`: String for the serial number of the result
    - `totalTimeInSeconds`: Float for the total time in seconds of the result
    - `partNumber`: String representing the part number of the result
    - `programName`: String of the program name
    - `startedAt`: ISO-8601 formatted UTC timestamp indicating when the result was started.
    - `updatedAt`: ISO-8601 formatted UTC timestamp indicating when the result was last updated.
    - `keywords`: A list of keyword strings
    - `properties`: A dictionary of additional string to string properties
    - `fileIds`: A list of string ids for files stored in the file service (`/nifile`)
    - `dataTableIds`: A list of string ids for data tables stored in the data frame service (`/nidataframe`)
    - `workspaceId`: String for the workspace identifier of the result
    See [Dynamic Linq](https://github.com/ni/systemlink-OpenAPI-documents/wiki/Dynamic-Linq-Query-Language)
    documentation for more details.
    `"@0"`, `"@1"` etc. can be used in conjunction with the `substitutions` parameter to keep this
    query string more simple and reusable.
    """

    substitutions: Optional[List[str]] = None
    """String substitutions into the `filter`.
    Makes substitutions in the query filter expression. Substitutions for the query expression are
    indicated by non-negative integers that are prefixed with the "at" symbol. Each substitution in
    the given expression will be replaced by the element at the corresponding index (zero-based) in
    this list. For example, "@0" in the filter expression will be replaced with the element at the
    zeroth index of the substitutions list.
    """


class QueryProductsBase(JsonModel):
    product_filter: Optional[str] = None
    """
    The product query filter in Dynamic Linq format.
    Allowed properties in the filter are:
    - `id`: String for the global identifier of the product
    - `partNumber`: String representing the part number of the product
    - `name`: String of the product name
    - `family`: String for the product family
    - `updatedAt`: ISO-8601 formatted UTC timestamp indicating when the product was last updated.
    - `keywords`: A list of keyword strings
    - `properties`: A dictionary of additional string to string properties
    - `fileIds`: A list of string ids for files stored in the file service (`/nifile`)
    See [Dynamic Linq](https://github.com/ni/systemlink-OpenAPI-documents/wiki/Dynamic-Linq-Query-Language)
    documentation for more details.
    `"@0"`, `"@1"` etc. can be used in conjunction with the `substitutions` parameter to keep this
    query string more simple and reusable.
    """

    product_substitutions: Optional[List[str]] = None
    """String substitutions into the `filter`.
    Makes substitutions in the query filter expression. Substitutions for the query expression are
    indicated by non-negative integers that are prefixed with the "at" symbol. Each substitution in
    the given expression will be replaced by the element at the corresponding index (zero-based) in
    this list. For example, "@0" in the filter expression will be replaced with the element at the
    zeroth index of the substitutions list.
    """


class QueryResultsRequest(QueryResultsBase, QueryProductsBase, WithPaging):

    order_by: Optional[ResultOrderByField] = Field(None, alias="orderBy")
    """Specifies the fields to use to sort the results.
    By default, results are sorted by `id`
    """
    order_by_key: Optional[str] = Field(None, alias="orderByKey")
    """Specifies the property to use to sort the results when ordering by PROPERTIES.
    Results that do not contain the orderByKey will be considered the smallest value.
    """
    order_by_comparison_type: Optional[ComparisonType] = Field(
        None, alias="orderByComparisonType"
    )
    """An enumeration of comparison types that can be used for ordered queries.
    For non-DEFAULT comparisons, values that cannot be converted will be considered the smallest value.
    """
    descending: Optional[bool] = None
    """Specifies whether to return the results in descending order.
    By default, this value is `false` and results are sorted in ascending order.
    """
    projection: Optional[List[ResultProjection]] = None
    """Specifies the fields to include in the returned results.
    Fields you do not specify are excluded. Returns all fields if no value is specified.
    """
    take: Optional[int] = None
    """Maximum number of results to return in the current API response.
    Uses the default if the specified value is negative. The default value is `1000` results.
    """
    return_count: Optional[bool] = None
    """If true, the response will include a count of all results matching the filter.
    By default, this value is `False` and count is not returned. Note that returning the count may
    incur performance penalties as the service may have to do a complete walk of the database to
    compute count. """


class QueryResultValuesRequest(QueryResultsBase):
    field: Optional[ResultField] = None
    """The result field to return for this query."""

    starts_with: Optional[str] = None
    """Only return string parameters prefixed by this value (case sensitive)."""



from enum import Enum
from typing import List, Optional

from nisystemlink.clients.core._uplink._with_paging import WithPaging


class TestPlanTemplateOrderBy(str, Enum):
    """An enumeration by which test plan templates can be ordered/sorted."""

    ID = "ID"
    NAME = "NAME"
    TEMPLATE_GROUP = "TEMPLATE_GROUP"
    CREATED_AT = "CREATED_AT"
    UPDATED_AT = "UPDATED_AT"


class TestPlanTemplateField(str, Enum):
    """Model for an object describing an test plan template with all of its properties."""

    __test__ = False

    ID = "ID"
    NAME = "NAME"
    TEMPLATE_GROUP = "TEMPLATE_GROUP"
    PRODUCT_FAMILIES = "PRODUCT_FAMILIES"
    PART_NUMBERS = "PART_NUMBERS"
    SUMMARY = "SUMMARY"
    DESCRIPTION = "DESCRIPTION"
    TEST_PROGRAM = "TEST_PROGRAM"
    ESTIMATED_DURATION_IN_SECONDS = "ESTIMATED_DURATION_IN_SECONDS"
    SYSTEM_FILTER = "SYSTEM_FILTER"
    DUT_FILTER = "DUT_FILTER"
    EXECUTION_ACTIONS = "EXECUTION_ACTIONS"
    FILE_IDS = "FILE_IDS"
    WORKSPACE = "WORKSPACE"
    PROPERTIES = "PROPERTIES"
    DASHBOARD = "DASHBOARD"
    CREATED_BY = "CREATED_BY"
    UPDATED_BY = "UPDATED_BY"
    CREATED_AT = "CREATED_AT"
    UPDATED_AT = "UPDATED_AT"


class QueryTestPlanTemplatesRequest(WithPaging):
    """Request information for the query test plan templates API."""

    filter: Optional[str] = None
    """The test plan template query filter in dynamic LINQ format.

    `id`: String representing the ID of a test plan template. Field supports only equals '=' and not
    equal '!=' operators for filtering.
    `productFamilies`: Array of strings representing the product families to which the test plan
    template belongs.
    `partNumbers`: Array of strings representing the part numbers of the products to which the test
    plan template belongs.
    `fileIds`: The array of file IDs associated with the test plan template.
    `name`: String representing the name of a test plan template.
    `summary`: String representing the summary of a test plan template.
    `description`: String representing description of the test plan created from this template.
    `templateGroup`: String representing the template group defined by the user.
    `testProgram`: String representing the test program name of the test plan created from this
    template.
    `systemFilter`: String representing the LINQ filter used to filter the potential list of systems
    `dutFilter`: String representing the LINQ filter used to filter the potential list of DUTs
    capable of executing test plans created from this template.
    `workspace`: String representing the workspace where the test plan template belongs.
    `createdBy`: String representing the user who created the test plan template.
    `updatedBy`: String representing the user who updated the test plan template.
    `createdAt`: ISO-8601 formatted timestamp indicating when the test plan template was created.
    `updatedAt`: ISO-8601 formatted timestamp indicating when the test plan template was most
    recently updated.
    `properties`: Collection of key-value pairs related to a test plan created from this template.
    Example: properties.Any(key == "Location" & value == "Austin")

    See [Dynamic Linq](https://github.com/ni/systemlink-OpenAPI-documents/wiki/Dynamic-Linq-Query-Language)
    documentation for more details.

    `"@0"`, `"@1"` etc. can be used in conjunction with the `substitutions` parameter to keep this
    query string more simple and reusable."""

    take: Optional[int] = None
    """The maximum number of test plan templates to return."""

    order_by: Optional[TestPlanTemplateOrderBy] = None
    """Field by which test plan templates can be ordered/sorted."""

    substitutions: Optional[List[str]] = None
    """Makes substitutions in the query filter expression
    using non-negative integers. These integers
    use the @ symbol as a prefix. The filter
    expression replaces each substitution
    with the element at the corresponding
    index in this list. The index is zero-based."""

    descending: Optional[bool] = None
    """Whether to return the test plan templates in the descending order. By default, test plan
    templates are sorted in the ascending order."""

    projection: Optional[List[TestPlanTemplateField]] = None
    """
    Gets or sets the projection to be used when retrieving the assets. If not specified,
    all properties will be returned.
    """



from typing import List, Optional

from nisystemlink.clients.core import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel
from nisystemlink.clients.testmonitor.models._create_steps_request import (
    CreateStepRequest,
)
from nisystemlink.clients.testmonitor.models._step import Step


class CreateStepsPartialSuccess(JsonModel):
    steps: List[Step]
    """The list of steps that were successfully created."""

    failed: Optional[List[CreateStepRequest]] = None
    """The list of step requests that failed.

    If this is `None`, then all steps were successfully created.
    """

    error: Optional[ApiError] = None
    """Error messages for steps that were not created.

    If this is `None`, then all steps were successfully created.
    """



from enum import Enum
from typing import List, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel
from pydantic import Field


class ProductOrderBy(str, Enum):
    """The valid ways to order a product query."""

    ID = "ID"
    FAMILY = "FAMILY"
    PART_NUMBER = "PART_NUMBER"
    NAME = "NAME"
    UPDATED_AT = "UPDATED_AT"


class ProductField(str, Enum):
    """An enumeration of product fields for which the values can be queried for."""

    ID = "ID"
    FAMILY = "FAMILY"
    PART_NUMBER = "PART_NUMBER"
    NAME = "NAME"
    UPDATED_AT = "UPDATED_AT"


class ProductProjection(str, Enum):
    """An enumeration of all fields in a Product. These are used to project the required fields
    from the API response.
    """

    ID = "ID"
    FAMILY = "FAMILY"
    PART_NUMBER = "PART_NUMBER"
    NAME = "NAME"
    UPDATED_AT = "UPDATED_AT"
    WORKSPACE = "WORKSPACE"
    KEYWORDS = "KEYWORDS"
    PROPERTIES = "PROPERTIES"
    FILE_IDS = "FILE_IDS"


class QueryProductsBase(JsonModel):
    filter: Optional[str] = None
    """
    The product query filter in Dynamic Linq format.

    Allowed properties in the filter are:
    - `id`: String for the global identifier of the product
    - `partNumber`: String representing the part number of the product
    - `name`: String of the product name
    - `family`: String for the product family
    - `updatedAt`: ISO-8601 formatted UTC timestamp indicating when the product was last updated.
    - `keywords`: A list of keyword strings
    - `properties`: A dictionary of additional string to string properties
    - `fileIds`: A list of string ids for files stored in the file service (`/nifile`)

    See [Dynamic Linq](https://github.com/ni/systemlink-OpenAPI-documents/wiki/Dynamic-Linq-Query-Language)
    documentation for more details.

    `"@0"`, `"@1"` etc. can be used in conjunction with the `substitutions` parameter to keep this
    query string more simple and reusable.
    """

    substitutions: Optional[List[str]] = None
    """String substitutions into the `filter`.

    Makes substitutions in the query filter expression. Substitutions for the query expression are
    indicated by non-negative integers that are prefixed with the "at" symbol. Each substitution in
    the given expression will be replaced by the element at the corresponding index (zero-based) in
    this list. For example, "@0" in the filter expression will be replaced with the element at the
    zeroth index of the substitutions list.
    """


class QueryProductsRequest(QueryProductsBase):

    order_by: Optional[ProductOrderBy] = Field(None, alias="orderBy")
    """Specifies the fields to use to sort the products.

    By default, products are sorted by `id`
    """

    descending: Optional[bool] = None
    """Specifies whether to return the products in descending order.

    By default, this value is `false` and products are sorted in ascending order.
    """

    projection: Optional[List[ProductProjection]] = None
    """Specifies the product fields to project.

    When a field value is given here, the corresponding field will be present in all returned products,
    and all unspecified fields will be excluded. If no projection is specified, all product fields
    will be returned.
    """

    take: Optional[int] = None
    """Maximum number of products to return in the current API response.

    Uses the default if the specified value is negative. The default value is `1000` products.
    """

    continuation_token: Optional[str] = None
    """Allows users to continue the query at the next product that matches the given criteria.

    To retrieve the next page of products, pass the continuation token from the previous
    page in the next request. The service responds with the next page of data and provides a new
    continuation token. To paginate results, continue sending requests with the newest continuation
    token provided in each response.
    """

    return_count: Optional[bool] = None
    """If true, the response will include a count of all products matching the filter.

    By default, this value is `False` and count is not returned. Note that returning the count may
    incur performance penalties as the service may have to do a complete walk of the database to
    compute count. """


class QueryProductValuesRequest(QueryProductsBase):
    field: Optional[ProductField] = None
    """The product field to return for this query."""

    starts_with: Optional[str] = None
    """Only return string parameters prefixed by this value (case sensitive)."""



from typing import List

from nisystemlink.clients.product._product_client import ProductClient
from nisystemlink.clients.product.models._paged_products import PagedProducts
from nisystemlink.clients.product.models._product import (
    Product,
)
from nisystemlink.clients.product.models._query_products_request import (
    QueryProductsRequest,
)


def get_products_linked_to_file(client: ProductClient, file_id: str) -> List[Product]:
    """Gets a list of all the products that are linked to the file.

    Args:
        `client` : The `ProductClient` to use for the request.
        `file_id`: The id of the file to query links for.

    Returns:
        `List[Product]`: A list of all the products that are linked to the file with `file_id`
    """
    query_request = QueryProductsRequest(
        filter=f'fileIds.Contains("{file_id}")', take=100
    )
    response: PagedProducts = client.query_products_paged(query_request)
    products = response.products
    while response.continuation_token:
        query_request.continuation_token = response.continuation_token
        response = client.query_products_paged(query_request)
        products.extend(response.products)
    return products



import io
from typing import List, Optional

from nisystemlink.clients import core
from nisystemlink.clients.core._api_error import ApiError
from nisystemlink.clients.core._uplink._base_client import BaseClient
from nisystemlink.clients.core._uplink._file_like_response import (
    file_like_response_handler,
)
from nisystemlink.clients.core._uplink._methods import (
    delete,
    get,
    post,
    put,
    response_handler,
)
from nisystemlink.clients.core.helpers._iterator_file_like import IteratorFileLike
from uplink import Part, Path, retry

from . import models


@retry(when=retry.when.status(429), stop=retry.stop.after_attempt(5))
class NotebookClient(BaseClient):
    def __init__(self, configuration: Optional[core.HttpConfiguration] = None):
        """Initialize an instance.

        Args:
            configuration: Defines the web server to connect to and information about
                how to connect. If not provided, the
                :class:`HttpConfigurationManager <nisystemlink.clients.core.HttpConfigurationManager>`
                is used to obtain the configuration.

        Raises:
            ApiException: if unable to communicate with the Notebook Service.
        """
        if configuration is None:
            configuration = core.HttpConfigurationManager.get_configuration()
        super().__init__(configuration, base_path="/")

    @get("ninotebook/v1/notebook/{id}")
    def get_notebook(self, id: str) -> models.NotebookMetadata:
        """Gets a notebook metadata by ID.

        Args:
            id: The ID of the notebook to read.

        Returns:
            The notebook metadata.

        Raises:
            ApiException: if unable to communicate with the ``/ninotebook`` service or provided invalid
                arguments.
        """
        ...

    @put("ninotebook/v1/notebook/{id}")
    def __update_notebook(
        self,
        id: Path,
        metadata: Part = None,
        content: Part = None,
    ) -> models.NotebookMetadata:
        """Updates a notebook metadata by ID.

        Args:
            id: The ID of the notebook to update.
            metadata: The notebook metadata.
            content: The notebook binary content.

        Returns:
            The updated notebook metadata.

        Raises:
            ApiException: if unable to communicate with the ``/ninotebook`` service or provided invalid
                arguments.
        """
        ...

    def update_notebook(
        self,
        id: str,
        metadata: Optional[models.NotebookMetadata] = None,
        content: Optional[io.BufferedReader] = None,
    ) -> models.NotebookMetadata:
        """Updates a notebook metadata by ID.

        Args:
            id: The ID of the notebook to update.
            metadata: The notebook metadata.
            content: The notebook binary content.

        Returns:
            The updated notebook metadata.

        Raises:
            ApiException: if unable to communicate with the ``/ninotebook`` service or provided invalid
                arguments.
        """
        metadata_io = None
        if metadata is not None:
            metadata_str = metadata.model_dump_json(by_alias=True, exclude_unset=True)
            metadata_io = io.BytesIO(metadata_str.encode("utf-8"))

        return self.__update_notebook(
            id=id,
            metadata=metadata_io,
            content=content,
        )

    @delete("ninotebook/v1/notebook/{id}")
    def delete_notebook(self, id: str) -> None:
        """Deletes a notebook by ID.

        Args:
            id: The ID of the notebook to delete.

        Raises:
            ApiException: if unable to communicate with the ``/ninotebook`` service or provided invalid
                arguments.
        """
        ...

    @post("ninotebook/v1/notebook")
    def __create_notebook(
        self,
        metadata: Part,
        content: Part,
    ) -> models.NotebookMetadata:
        """Creates a new notebook.

        Args:
            metadata: The notebook metadata.
            content: The notebook binary content.

        Returns:
            The created notebook metadata.

        Raises:
            ApiException: if unable to communicate with the ``/ninotebook`` service or provided invalid
                arguments.
        """
        ...

    def create_notebook(
        self,
        metadata: models.NotebookMetadata,
        content: io.BufferedReader,
    ) -> models.NotebookMetadata:
        """Creates a new notebook.

        Args:
            metadata: The notebook metadata.
            content: The notebook binary content.

        Returns:
            The created notebook metadata.

        Raises:
            ApiException: if unable to communicate with the ``/ninotebook`` service or provided invalid
                arguments.
        """
        metadata_str = metadata.model_dump_json()

        metadata_io = io.BytesIO(metadata_str.encode("utf-8"))
        return self.__create_notebook(
            metadata=metadata_io,
            content=content,
        )

    @post("ninotebook/v1/notebook/query")
    def query_notebooks(
        self, query: models.QueryNotebookRequest
    ) -> models.PagedNotebooks:
        """Queries notebooks.

        Args:
            query: The query parameters.

        Returns:
            The paged notebooks.

        Raises:
            ApiException: if unable to communicate with the ``/ninotebook`` service or provided invalid
                arguments.
        """
        ...

    @response_handler(file_like_response_handler)
    @get("ninotebook/v1/notebook/{id}/content")
    def get_notebook_content(self, id: str) -> IteratorFileLike:
        """Gets a notebook content by ID.

        Args:
            id: The ID of the notebook to read.

        Returns:
            A file-like object for reading the notebook content.

        Raises:
            ApiException: if unable to communicate with the ``/ninotebook`` service or provided invalid
                arguments.
        """
        ...

    @post("ninbexecution/v1/executions")
    def create_executions(
        self, executions: List[models.CreateExecutionRequest]
    ) -> models.CreateExecutionsResponse:
        """Create one or more executions of Jupyter notebooks.

        Args:
            execution: information about an execution of a Jupyter notebook.

        Returns:
            A response to a request to create executions.

        Raises:
            ApiException: if unable to communicate with the ``/ninbexecution`` Service
                or provided an invalid argument.
        """
        ...

    @get("ninbexecution/v1/executions/{id}")
    def get_execution_by_id(self, id: str) -> models.Execution:
        """Get information about the specified execution of a Jupyter notebook.

        Args:
            id: the ID of the execution.

        Returns:
            Information about the execution of a Jupyter notebook fetched using it's Id.

        Raises:
            ApiException: if unable to communicate with the ``/ninbexecution`` Service
                or provided an invalid argument.
        """
        ...

    @post("ninbexecution/v1/query-executions")
    def __query_executions(
        self, query: models._QueryExecutionsRequest
    ) -> List[models.Execution]:
        """Query executions of Jupyter notebooks.

        Args:
            query: query for executions of Jupyter notebooks.

        Returns:
            A response to a request to query executions.

        Raises:
            ApiException: if unable to communicate with the ``/ninbexecution`` Service
                or provided an invalid argument.
        """
        ...

    def query_executions(
        self, query: models.QueryExecutionsRequest
    ) -> List[models.Execution]:
        """Query executions of Jupyter notebooks.

        Args:
            query: query for executions of Jupyter notebooks.

        Returns:
            A response to a request to query executions.

        Raises:
            ApiException: if unable to communicate with the ``/ninbexecution`` Service
                or provided an invalid argument.
        """
        projection = ",".join(query.projection)
        projection_str = f"new({projection})" if projection else None

        query_params = {
            "filter": query.filter,
            "order_by": query.order_by,
            "descending": query.descending,
            "projection": projection_str,
        }

        query_params = {k: v for k, v in query_params.items() if v is not None}

        query_request = models._QueryExecutionsRequest(**query_params)

        return self.__query_executions(query=query_request)

    @post("ninbexecution/v1/retry-executions", return_key="error")
    def retry_executions(self, ids: List[str]) -> Optional[ApiError]:
        """Retries existing executions based on failed, canceled or timed-out executions.

        Args:
            ids: List of execution IDs to retry.

        Returns:
            An ApiError object if executions could not be retried.
            None if executions were retried successfully.

        Raises:
            ApiException: if unable to communicate with the ``/ninbexecution`` Service
                or provided an invalid argument.
        """
        ...

    @post("ninbexecution/v1/cancel-executions", return_key="error")
    def cancel_executions(self, ids: List[str]) -> Optional[ApiError]:
        """Cancel queued and in-progress executions.

        Args:
            ids: List of execution IDs to cancel.

        Returns:
            An ApiError object if executions could not be canceled.
            None if executions were canceled successfully.

        Raises:
            ApiException: if unable to communicate with the ``/ninbexecution`` Service
                or provided an invalid argument.
        """
        ...

    @post("ninbexecution/v1/create-executions-from-existing")
    def create_executions_from_existing(
        self, ids: List[str]
    ) -> models.CreateExecutionsResponse:
        """Create new executions based on already existing succeeded executions.

        Args:
            ids: List of execution IDs to run again.

        Returns:
            A response to a request to create executions.

        Raises:
            ApiException: if unable to communicate with the ``/ninbexecution`` Service
                or provided an invalid argument.
        """
        ...



from typing import List, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._schedule_test_plan_request import ScheduleTestPlanRequest


class ScheduleTestPlansRequest(JsonModel):
    """Represents the request body for scheduling multiple test plans."""

    test_plans: List[ScheduleTestPlanRequest]
    """List of test plan scheduling content objects."""

    replace: Optional[bool] = None
    """(Optional) If true, replaces existing scheduled test plans."""



from ._notebook_metadata import NotebookMetadata
from ._query_notebook_request import QueryNotebookRequest
from ._query_notebook_response import PagedNotebooks
from ._query_execution_request import (
    ExecutionField,
    ExecutionSortField,
    _QueryExecutionsRequest,
    QueryExecutionsRequest,
)
from ._create_execution_request import (
    CreateExecutionRequest,
)
from ._create_execution_response import (
    CreatedExecution,
    CreateExecutionsResponse,
)
from ._execution import (
    Execution,
    SourceType,
    ReportType,
    ReportSettings,
    ExecutionPriority,
    ExecutionResourceProfile,
    ExecutionStatus,
    ExecutionErrorCode,
)

# flake8: noqa



from nisystemlink.clients.core._uplink._json_model import JsonModel


class Operation(JsonModel):
    """Represents an operation that can be performed on a data frame."""

    available: bool
    """Whether or not the operation is available to the caller (e.g. due to permissions)."""

    version: int
    """The version of the available operation."""



from datetime import datetime
from enum import Enum
from typing import Any, Dict, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel
from pydantic import Field


class SourceType(str, Enum):
    """Source type of an execution"""

    MANUAL = "MANUAL"

    TRIGGERED = "TRIGGERED"


class Source(JsonModel):
    """An object that defines properties set by routine service"""

    type: SourceType
    """Source type of an execution"""

    routine_id: Optional[str] = None
    """ID of the routine that triggered the execution"""

    event_id: Optional[str] = None
    """Unique identifier of event that triggered the execution"""


class ReportType(str, Enum):
    """Available types for a report that is going to be generated."""

    NO_REPORT = "NO_REPORT"

    HTML = "HTML"

    PDF = "PDF"


class ReportSettings(JsonModel):
    """A class that defines settings of the Report"""

    format: ReportType
    """Type for the report that is going to be generated."""

    exclude_code: bool
    """Boolean parameter that will define if the source code should be included in the report or not."""


class ExecutionPriority(str, Enum):
    """Execution priority. Can be one of Low, Medium or High."""

    LOW = "LOW"

    MEDIUM = "MEDIUM"

    HIGH = "HIGH"


class ExecutionResourceProfile(str, Enum):

    DEFAULT = "DEFAULT"

    LOW = "LOW"

    MEDIUM = "MEDIUM"

    HIGH = "HIGH"


class ExecutionStatus(str, Enum):
    """Status of an execution."""

    IN_PROGRESS = "IN_PROGRESS"

    QUEUED = "QUEUED"

    FAILED = "FAILED"

    SUCCEEDED = "SUCCEEDED"

    CANCELED = "CANCELED"

    TIMED_OUT = "TIMED_OUT"


class ExecutionErrorCode(str, Enum):
    """Execution error code."""

    NO_ERROR = "NO_ERROR"

    NOTEBOOK_ERROR = "NOTEBOOK_ERROR"

    NOTEBOOK_TIMEOUT_ERROR = "NOTEBOOK_TIMEOUT_ERROR"

    NOTEBOOK_NOT_FOUND_ERROR = "NOTEBOOK_NOT_FOUND_ERROR"

    NOTEBOOK_RESULT_TOO_BIG_ERROR = "NOTEBOOK_RESULT_TOO_BIG_ERROR"

    NOT_PUBLISHED_ERROR = "NOT_PUBLISHED_ERROR"

    OUT_OF_MEMORY_ERROR = "OUT_OF_MEMORY_ERROR"

    UNKNOWN_ERROR = "UNKNOWN_ERROR"

    DEAD_KERNEL_ERROR = "DEAD_KERNEL_ERROR"

    EXECUTION_COULD_NOT_BE_RETRIED = "EXECUTION_COULD_NOT_BE_RETRIED"


class Execution(JsonModel):
    """Information about an execution of a Jupyter notebook that has the cachedResult field added."""

    id: Optional[str] = None
    """The ID of the execution."""

    notebook_id: Optional[str] = None
    """The ID of the executed notebook."""

    organization_id: Optional[str] = Field(None, alias="orgId")
    """The org ID of the user creating the request."""

    user_id: Optional[str] = None
    """The user ID of the user creating the request."""

    parameters: Optional[Dict[str, Any]] = None
    """The input parameters for this execution of the notebook. The keys are strings and the values can be of any
    valid JSON type."""

    workspace_id: Optional[str] = None
    """The ID of the workspace this execution belongs to."""

    timeout: Optional[int] = None
    """The number of seconds the execution runs before it aborts if uncompleted. The timer starts once status is
    IN_PROGRESS. 0 means infinite."""

    status: Optional[ExecutionStatus] = None
    """Status of an execution."""

    queued_at: Optional[datetime] = None
    """Timestamp of when the notebook execution was queued."""

    started_at: Optional[datetime] = None
    """Timestamp of when the notebook execution was started."""

    completed_at: Optional[datetime] = None
    """Timestamp of when the notebook execution was completed."""

    last_updated_timestamp: Optional[datetime] = None
    """Timestamp of when the notebook execution was last updated."""

    last_updated_by: Optional[str] = None
    """"The user ID of the user who last updated the execution."""

    retry_count: Optional[int] = None
    """The number of manually retried attempts of the notebook execution."""

    exception: Optional[str] = None
    """Exception that occured during the execution. This is used only when status is FAILED."""

    error_code: Optional[ExecutionErrorCode] = None
    """Execution error code."""

    report_id: Optional[str] = None
    """The ID of the report this execution generates."""

    report_settings: Optional[ReportSettings] = None
    """Settings of the Report"""

    result: Optional[Dict[str, Optional[object]]] = None
    """Result of the execution. This is used only when status is SUCCEEDED."""

    source: Optional[Source] = None
    """An object that defines properties set by routine service"""

    priority: Optional[ExecutionPriority] = None
    """Execution priority. Can be one of Low, Medium or High."""

    resource_profile: Optional[ExecutionResourceProfile] = None
    """Resource profile of the execution."""



from typing import Dict, List, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._dashboard import Dashboard
from ._execution_definition import ExecutionDefinition


class CreateTestPlanRequest(JsonModel):
    """Represents the request body content for creating a test plan."""

    name: Optional[str] = None
    """The name of the test plan."""

    state: Optional[str] = None
    """The state of the test plan."""

    template_id: Optional[str] = None
    """The ID of the template to use for the test plan."""

    description: Optional[str] = None
    """A description of the test plan."""

    assigned_to: Optional[str] = None
    """The user or group assigned to the test plan."""

    part_number: Optional[str] = None
    """The part number associated with the test plan."""

    dut_id: Optional[str] = None
    """The Device Under Test (DUT) ID."""

    dut_serial_number: Optional[str] = None
    """The Device Under Test (DUT) serial number."""

    test_program: Optional[str] = None
    """The test program associated with the test plan."""

    work_order_id: Optional[str] = None
    """The work order ID associated with the test plan."""

    estimated_duration_in_seconds: Optional[int] = None
    """The estimated duration of the test plan in seconds."""

    system_filter: Optional[str] = None
    """The system filter to apply."""

    dut_filter: Optional[str] = None
    """The DUT filter to apply."""

    execution_actions: Optional[List[ExecutionDefinition]] = None
    """List of execution actions for the test plan."""

    file_ids_from_template: Optional[List[str]] = None
    """List of file IDs from the template."""

    workspace: Optional[str] = None
    """The workspace associated with the test plan."""

    properties: Optional[Dict[str, str]] = None
    """Additional properties for the test plan."""

    dashboard: Optional[Dashboard] = None
    """The dashboard associated with the test plan."""



from typing import Annotated, Any, Dict, List, Literal, Optional, Union

from nisystemlink.clients.core._uplink._json_model import JsonModel
from pydantic import Field


class Job(JsonModel):
    """Represents a job to be executed, including its functions, arguments, and metadata."""

    functions: List[str]
    """List of function names to execute."""

    arguments: List[List[Any]]
    """List of argument lists for each function."""

    metadata: Dict[str, Any]
    """Additional metadata for the job."""


class NotebookExecution(JsonModel):
    """Defines the execution of a notebook."""

    action: str
    """User defined action to perform in workflow (user defined)."""

    type: Literal["NOTEBOOK"] = Field(default="NOTEBOOK")
    """Type of execution, default is 'NOTEBOOK'."""

    notebookId: str
    """ID of the notebook to execute."""

    parameters: Optional[Dict[str, str]] = None
    """	Dictionary of parameters that will be passed to the notebook when the execution is run."""


class ManualExecution(JsonModel):
    """Represents a manual execution definition."""

    action: str
    """User defined action to perform in workflow (user defined)."""

    type: Literal["MANUAL"] = Field(default="MANUAL")
    """Type of execution, default is 'MANUAL'."""


class JobExecution(JsonModel):
    """Defines the execution of one or more jobs."""

    action: str
    """User defined action to perform in workflow (user defined)."""

    type: Literal["JOB"] = Field(default="JOB")
    """Type of execution, default is 'JOB'."""

    jobs: Optional[List[Job]] = None
    """List of jobs to execute."""

    systemId: Optional[str] = None
    """Optional system ID where jobs will run."""


class NoneExecution(JsonModel):
    """Represents a definition where no execution is specified."""

    action: str
    """User defined action to perform in workflow (user defined)."""

    type: Literal["NONE"] = Field(default="NONE")
    """Type of execution, default is 'NONE'."""


ExecutionDefinition = Annotated[
    Union[
        NotebookExecution,
        ManualExecution,
        JobExecution,
        NoneExecution,
    ],
    Field(discriminator="type"),
]



from typing import List, Optional

from nisystemlink.clients.core._uplink._json_model import JsonModel

from ._asset import Asset


class QueryAssetsResponse(JsonModel):
    """Model for assets Response containing the assets satisfying the query and the total count of matching assets."""

    assets: Optional[List[Asset]] = None
    """Gets or sets array of assets."""

    total_count: Optional[int] = None
    """Gets or sets the total number of Assets which match the query."""



from typing import List, Optional

from nisystemlink.clients.core import ApiError
from nisystemlink.clients.core._uplink._json_model import JsonModel
from nisystemlink.clients.testmonitor.models._step import Step
from nisystemlink.clients.testmonitor.models._update_steps_request import (
    UpdateStepRequest,
)


class UpdateStepsPartialSuccess(JsonModel):
    steps: List[Step]
    """The list of steps that were successfully updated."""

    failed: Optional[List[UpdateStepRequest]] = None
    """The list of steps that were not updated.

    If this is `None`, then all steps were successfully updated.
    """

    error: Optional[ApiError] = None
    """Error messages for steps that were not updated.

    If this is `None`, then all steps were successfully updated.
    """



from typing import List, Optional

from nisystemlink.clients.core._uplink._with_paging import WithPaging

from ._test_plan_templates import TestPlanTemplate


class PagedTestPlanTemplates(WithPaging):
    """The response containing the list of products, total count of products and the continuation
    token if applicable.
    """

    test_plan_templates: List[TestPlanTemplate]
    """A list of all the products in this page."""

    total_count: Optional[int] = None
    """The total number of products that match the query."""
